from _typeshed import Incomplete
from common_utils.logger import create_logger as create_logger

class FirebaseClient:
    log: Incomplete
    headers: Incomplete
    database_url: Incomplete
    def __init__(self, realtime_db_url: str | None = None, env_name: str = 'FIREBASE_REALTIME_DB_URL') -> None: ...
    def get_list(self, ref: str, max_results: int = 100, convert_to_list: bool = True): ...
    def get(self, ref: str): ...
    def set(self, ref: str, data: dict): ...
    def delete(self, ref: str): ...
    def get_entry(self, *args, **kwargs): ...
    def set_entry(self, *args, **kwargs): ...
    def delete_entry(self, *args, **kwargs): ...
    def update(self, ref: str, data: dict): ...



from datetime import datetime, timezone, timedelta


def get_datetime_now_utc_millisecond() -> str:
    return datetime.now(timezone.utc).strftime("%Y-%m-%dT%H:%M:%S") + ".000+0000"


def get_timestamp_from_offset(days_offset: int) -> int:
    """ Returns the unit-time timestamp of now minus the offset"""
    offset_date = datetime.now() - timedelta(days=days_offset)
    return int(offset_date.timestamp())


import os
import threading
os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = "TRUE"
import pygame

from common_utils.config import ROOT_DIR


class SoundHandler:
    def __init__(self, default_volume):
        self.default_volume = default_volume

    def play_sound(self, file_path, volume=None):
        """Play sound in a separate thread"""
        thread = threading.Thread(target=self._play_sound, args=(file_path, volume))
        thread.start()

    def _play_sound(self, file_path, volume=None):
        volume = volume or self.default_volume
        pygame.mixer.init()
        pygame.mixer.music.set_volume(volume)
        pygame.mixer.music.load(filename=file_path)
        pygame.mixer.music.play()
        while pygame.mixer.music.get_busy():
            continue
        pygame.mixer.quit()




if __name__ == '__main__':
    sound_handler = SoundHandler(default_volume=1)
    sound_handler.play_sound(f'{ROOT_DIR}res/start-sound.mp3')



from _typeshed import Incomplete
from common_utils.apis.ticktick.cookies_login import get_authenticated_ticktick_headers as get_authenticated_ticktick_headers
from common_utils.logger import create_logger as create_logger
from common_utils.time_utils import get_timestamp_from_offset as get_timestamp_from_offset
from datetime import datetime
from pydantic import BaseModel

class TickTickFocusTime(BaseModel):
    id: str
    type: int
    start_time: str | datetime
    end_time: str | datetime
    status: int
    pause_duration: int
    total_duration: int | None
    @staticmethod
    def to_camel(field_name: str) -> str: ...
    model_config: Incomplete

class TicktickFocusHandler:
    log: Incomplete
    headers: Incomplete
    raise_exceptions: Incomplete
    def __init__(self, cookies_path: str | None = None, always_raise_exceptions: bool = False, headless: bool = True, undetected: bool = False, download_driver: bool = False, username_env: str = 'TICKTICK_EMAIL', password_env: str = 'TICKTICK_PASSWORD') -> None: ...
    def get_all_focus_times(self, to_timestamp: int | None = None, days_offset: int | None = None) -> list[TickTickFocusTime] | None: ...



from _typeshed import Incomplete
from common_utils.config import ROOT_DIR as ROOT_DIR
from common_utils.logger import create_logger as create_logger

class OAuth2Data:
    client_id_env: str
    client_secret_env: str
    redirect_uri_env: str
    scope_env: str
    authorize_url: str
    token_url: str
    state: str | None

class OAuth2Handler:
    log: Incomplete
    auth_code: Incomplete
    client_id: Incomplete
    client_secret: Incomplete
    redirect_uri: Incomplete
    scope: Incomplete
    authorize_url: Incomplete
    token_url: Incomplete
    state: Incomplete
    port: Incomplete
    cache_path: Incomplete
    cache_name: Incomplete
    def __init__(self, data: OAuth2Data, cache_path: str | None = None) -> None: ...
    def callback_app(self, environ, start_response): ...
    def start_temp_server(self) -> None: ...
    def save_token(self, token) -> None: ...
    def load_token(self): ...
    def get_token(self): ...



import requests
from datetime import datetime, timedelta
from typing import Literal
from pydantic import BaseModel, ConfigDict

from common_utils.logger import create_logger
from common_utils.apis.ticktick.cookies_login import get_authenticated_ticktick_headers
from common_utils.web.api_request import post_request
from common_utils.time_utils import get_datetime_now_utc_millisecond


# TODO: dataclass for habit metadata


class TickTickHabitEntry(BaseModel):
    habit_id: str
    checkin_stamp: int
    goal: float | int
    value: float | int
    status: Literal[0, 1, 2]
    id: str | None = None
    checkin_time: str | None = None
    op_time: str | None = None

    @classmethod
    def init(
            cls,
            habit_id: str,
            date_stamp: int,
            habit_goal: int,
            status: Literal[0, 1, 2] | None = None,
            value: int | float | None = None,
    ) -> "TickTickHabitEntry":

        assert not (status and value), "You can only provide status or value, not both"
        assert status is not None or value is not None, "You need to provide either status or value"

        if value is None:
            value = habit_goal if status == 2 else 0
        if status is None:
            status = 2 if value >= habit_goal else 0

        now = get_datetime_now_utc_millisecond()
        return cls(
            checkin_stamp=date_stamp,
            checkin_time=now,
            goal=habit_goal,
            habit_id=habit_id,
            op_time=now,
            status=status,
            value=value,
        )

    @staticmethod
    def to_camel(field_name: str) -> str:
        """
        Convert a snake_case field name into camelCase.
        E.g. 'checkin_stamp' -> 'checkinStamp'
        """
        parts = field_name.split('_')
        return parts[0] + ''.join(word.capitalize() for word in parts[1:])

    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
        extra="allow"
    )




class TicktickHabitHandler:
    """Class that accesses the TickTick habits API. Used to post habit checkins.

    Definitions:
    - Habit: A habit is a task that can be checked in multiple times a day, with a goal value.
             Habits have a unique name and id, stored in their metadata.
    - Checkin: A checkin is a single entry of a habit, with a status and a value.
    """

    status_codes = {0: "Not completed", 1: "Failed", 2: "Completed"}
    url_habits = "https://api.ticktick.com/api/v2/habits"
    url_batch_checkin = "https://api.ticktick.com/api/v2/habitCheckins/batch"
    url_query_checkin = "https://api.ticktick.com/api/v2/habitCheckins/query"


    def __init__(
            self,
            cookies_path: str | None = None,
            always_raise_exceptions: bool = False,
            headless: bool = True,
            undetected: bool = False,
            download_driver: bool = False,
            username_env: str = 'TICKTICK_EMAIL',
            password_env: str = 'TICKTICK_PASSWORD',
    ):
        self.log = create_logger("Ticktick Habits")
        self.headers = get_authenticated_ticktick_headers(
            cookies_path=cookies_path,
            username_env=username_env,
            password_env=password_env,
            headless=headless,
            undetected=undetected,
            download_driver=download_driver,
        )
        self.habits, self.habit_ids = self._get_all_habits_metadata()
        self.raise_exceptions = always_raise_exceptions

    def _get_all_habits_metadata(self) -> tuple[dict, dict]:
        """Get the metadata of all habits and their ids"""

        habit_data = requests.get(url=self.url_habits, headers=self.headers).json()
        if "errorId" in habit_data:
            error_message = f"Error loading habits: {habit_data}"
            self.log.error(error_message)
            raise ValueError(error_message)

        habits_metadata = {habit["id"]: habit for habit in habit_data if 'id' in habit}
        habit_name_to_id_mapping = {habit["name"]: habit["id"] for habit in habit_data}
        return habits_metadata, habit_name_to_id_mapping

    def _post_habit_metadata(self, habit_id):
        # todo: update a single habit metadata, to update checkins quickly
        raise NotImplementedError

    def _init_habit_entry(
            self,
            habit_name: str,
            date_stamp: int,
            status: Literal[0, 1, 2] | None = None,
            value: int | None = None,
    ) -> TickTickHabitEntry:
        """Collect all data needed for a single habit checkin."""

        habit_id = self.habit_ids[habit_name]
        habit_goal = int(self.habits[habit_id]["goal"])
        return TickTickHabitEntry.init(
            habit_id=habit_id,
            date_stamp=date_stamp,
            status=status,
            value=value,
            habit_goal=habit_goal,
        )

    def post_checkin(
            self,
            habit_name: str,
            date_stamp: int,
            status: int | None = None,
            value: int | None = None,
            raise_exception: bool = False,
         ) -> None:
        """Post a single habit checkin to the TickTick API.

        Args:
            habit_name: Name of the habit to check-in
            date_stamp: Date of the check-in, in the format YYYYMMDD
            status: Status of the check-in. 0: Not completed, 1: Failed, 2: Completed
            value: The value amount to check in. for habits who require multiple units
            raise_exception: Flag to raise exception in case of an error
        """

        checkin_entry = self._init_habit_entry(habit_name, date_stamp, status, value)
        self.log.info(f"Checking {habit_name} on {date_stamp} as {status}: {value}/{checkin_entry.goal}")

        # create payload depending on if a checkin for that day already exists
        existing_checkin_entry = self.get_checkin(checkin_entry.habit_id, date_stamp)
        payload: dict[str, list[dict[str, str | int]]] = {"add": [], "update": [], "delete": []}

        if existing_checkin_entry:
            checkin_entry.id = existing_checkin_entry.id
            payload["update"].append(checkin_entry.model_dump())
        else:
            payload["add"].append(checkin_entry.model_dump())

        response = post_request(url="batch_checkin", payload=payload, headers=self.headers)
        if response:
            self.log.info(f"Checkin for {habit_name} successful with response: {response}")
        elif raise_exception or self.raise_exceptions:
            raise ValueError(f"Error posting Habit Checkin: {response}")

    def get_checkin(
            self,
            habit_id: str,
            date_stamp: int,
            raise_exception: bool = False,
    ) -> TickTickHabitEntry | None:
        """
        Retrieve a single checkin entry for a habit on a specific date, or None if not found
        """

        date = datetime.strptime(str(date_stamp), "%Y%m%d")
        after_stamp = (date - timedelta(days=1)).strftime("%Y%m%d")
        after_stamp = int(after_stamp)  # type: ignore
        payload = {"habitIds": [habit_id], "afterStamp": after_stamp}
        response = post_request(url=self.url_query_checkin, payload=payload, headers=self.headers)

        if response is None or response.get('checkins', None) is None:
            error_msg = f"No or malformed response from TickTick API for get_checkin: {response}"
            self.log.error(error_msg)
            if raise_exception or self.raise_exceptions:
                raise ValueError(error_msg)
            return None

        all_entries = response.get('checkins', {})
        habit_entries = all_entries.get(habit_id, [])
        for entry in habit_entries:
            if entry.get("checkinStamp", -1) == int(date_stamp):
                habit_entry = TickTickHabitEntry(**entry)
                return habit_entry

        return None

    def get_all_checkins(
            self,
            after_stamp: int = 19700101,
            habit_names: list[str] | str | None = None,
            raise_exception: bool = False
    ) -> dict[str, list[TickTickHabitEntry]] | None:
        """Get all checkins of all habits (or those provided), after a specific date stamp."""

        if not habit_names:
            habits_ids = list(self.habit_ids.values())
        else:
            habit_names = [habit_names] if isinstance(habit_names, str) else habit_names
            habits_ids = [self.habit_ids[habit] for habit in habit_names]

        payload = {"habitIds": habits_ids, "afterStamp": after_stamp}
        response = post_request(url=self.url_query_checkin, payload=payload, headers=self.headers)

        if response is None or response.get('checkins', None) is None:
            error_msg = f"No or malformed response from TickTick API for get_all_checkins: {response}"
            self.log.error(error_msg)
            if raise_exception:
                raise ValueError(error_msg)
            return None

        all_habits_entries = response.get("checkins", {})
        all_habits_entries_parsed = {}
        for habit_id, habits_entries in all_habits_entries.items():
            habit_name = self.habits[habit_id]["name"]
            habits_entries_objs = [TickTickHabitEntry(**entry, habitName=habit_name) for entry in habits_entries]
            all_habits_entries_parsed[habit_id] = habits_entries_objs

        return all_habits_entries_parsed


if __name__ == "__main__":
    from dotenv import load_dotenv
    load_dotenv()
    handlers = TicktickHabitHandler(always_raise_exceptions=True)
    single_checkin = handlers.get_checkin(habit_id='64f08ac16fc6ff16c2d1f3eb', date_stamp=20250520)
    checkins = handlers.get_all_checkins(after_stamp=20220101)
    print(checkins)


from _typeshed import Incomplete
from common_utils.logger import create_logger as create_logger
from dataclasses import dataclass
from selenium import webdriver
from time import sleep
from typing import Literal

@dataclass
class DriverAction:
    action: Literal['url', 'get', 'sleep', 'click', 'send_keys', 'get_text', 'get_texts', 'get_class', 'get_classes', 'get_href', 'get_hrefs', 'scroll_down', 'save_url', 'save_all_tabs_urls', 'save_last_tab_url', 'close_tab']
    result_key: str | None = ...
    value: str | int | float | None = ...
    css_identifier: str | None = ...
    delim: str | None = ...
    delim_idx: int | None = ...

class SeleniumHandler:
    log: Incomplete
    headless: Incomplete
    undetected: Incomplete
    verbose: Incomplete
    raise_exceptions: Incomplete
    download_driver: Incomplete
    def __init__(self, headless: bool = False, undetected: bool = False, verbose: bool = True, raise_exceptions: bool = True, download_driver: bool = False) -> None: ...
    def log_msg(self, message: str, level: Literal['debug', 'info', 'warning', 'error'] = 'info'): ...
    def get_driver(self): ...
    def run_actions(self, actions: list[DriverAction], driver: webdriver.Chrome | None = None) -> dict[str, str | list[str] | None]: ...
    def run_action(self, action: DriverAction, driver: webdriver.Chrome) -> str | list[str] | None: ...



from _typeshed import Incomplete
from common_utils.logger import create_logger as create_logger

log: Incomplete

def get_request(url, headers: Incomplete | None = None, catch_exception: bool = False) -> dict | None: ...
def post_request(url, payload, headers: Incomplete | None = None, raise_exception: bool = False) -> dict | None: ...



import requests
from pydantic import BaseModel, ConfigDict

from common_utils.apis.ticktick.cookies_login import get_authenticated_ticktick_headers
from common_utils.logger import create_logger


class TickTickTask(BaseModel):
    id: str
    project_id: str
    title: str
    status: int
    priority: int
    deleted: int
    created_time: str
    creator: int
    items: list
    column_id: str | None = None
    is_all_day: bool | None = None
    start_date: str | None = None
    due_date: str | None = None
    content: str | None = None

    @staticmethod
    def to_camel(field_name: str) -> str:
        """
        Convert a snake_case field name into camelCase.
        E.g. 'checkin_stamp' -> 'checkinStamp'
        """
        parts = field_name.split('_')
        return parts[0] + ''.join(word.capitalize() for word in parts[1:])

    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
        extra="allow"
    )



class TicktickTaskHandler:
    log = create_logger("TickTick Task Handler")
    url_get_tasks = 'https://api.ticktick.com/api/v2/batch/check/0'

    def __init__(
            self,
            cookies_path: str | None = None,
            always_raise_exceptions: bool = False,
            headless: bool = True,
            undetected: bool = False,
            download_driver: bool = False,
            username_env: str = 'TICKTICK_EMAIL',
            password_env: str = 'TICKTICK_PASSWORD',
    ):
        self.headers = get_authenticated_ticktick_headers(
            cookies_path=cookies_path,
            username_env=username_env,
            password_env=password_env,
            headless=headless,
            undetected=undetected,
            download_driver=download_driver,
        )
        self.raise_exceptions = always_raise_exceptions

    def get_all_tasks(self) -> list[TickTickTask] | None:
        """
        Get all TickTick tasks

        Returns:
            List of TickTickTask pydantic BaseModel objects. Objects can be converted via .dict()
        """
        response = requests.get(url=self.url_get_tasks, headers=self.headers).json()
        tasks_data = response.get('syncTaskBean', {}).get('update', None)
        if tasks_data is None:
            self.log.error("Getting Tasks failed")
            if self.raise_exceptions:
                raise ValueError("Getting Tasks failed")
            return None

        tasks = []
        for data in tasks_data:
            tasks.append(TickTickTask(**data))
        # tasks = [TickTickTask(**task_data) for task_data in tasks_data]

        return tasks


if __name__ == '__main__':
    TicktickTaskHandler().get_all_tasks()


import os
from common_utils.config import ROOT_DIR
from common_utils.logger import create_logger
from common_utils.web.cookies_handler import CookiesManager, LoginData, LoginSelectors


def get_authenticated_ticktick_headers(
        cookies_path: str | None = None,
        username_env: str = "TICKTICK_EMAIL",
        password_env: str = "TICKTICK_PASSWORD",
        headless: bool = True,
        undetected: bool = False,
        download_driver: bool = False,
):
    log = create_logger("TickTick Cookies Helper")

    if os.getenv(username_env) is None or os.getenv(password_env) is None:
        log.error(f"Either username or password environment variables {username_env} | {password_env} is None")

    cookies_manager = CookiesManager(
        login_data=LoginData(
            username_env=username_env,
            password_env=password_env,
            sign_in_url="https://www.ticktick.com/signin",
            selectors=LoginSelectors(
                username='input[placeholder="Email"]',
                password='#password',
                login_button='#app div[class^=body] button'
            )
        ),
        test_cookies_url="https://api.ticktick.com/api/v2/habits",
        test_cookies_response_fn=lambda data: "errorCode" not in data,
        min_num_cookies=5,
        headless=headless,
        undetected=undetected,
        download_driver=download_driver,
    )
    if cookies_path:
        cookies_manager.cookies_path = cookies_path
    elif ROOT_DIR:
        cookies_manager.cookies_path = f'{ROOT_DIR}/.ticktick-cookies.json'
    else:
        cookies_manager.cookies_path = '../../.ticktick-cookies.json'

    headers = cookies_manager.get_headers_with_cookies()

    return headers


from _typeshed import Incomplete
from common_utils.logger import create_logger as create_logger
from common_utils.web.selenium import SeleniumHandler as SeleniumHandler
from dataclasses import dataclass
from selenium import webdriver as webdriver

@dataclass
class LoginSelectors:
    username: str
    password: str
    login_button: str
    popup_close: str | None = ...

@dataclass
class LoginData:
    username_env: str
    password_env: str
    sign_in_url: str
    selectors: LoginSelectors

class CookiesManager:
    login_wait_time: int
    headers: Incomplete
    login_data: Incomplete
    test_cookies_url: Incomplete
    test_cookies_response_fn: Incomplete
    min_num_cookies: Incomplete
    log: Incomplete
    cookies_path: Incomplete
    selenium_handler: Incomplete
    def __init__(self, login_data: LoginData, test_cookies_url: str, test_cookies_response_fn: callable, min_num_cookies: int | None = None, cookies_path: str | None = None, headless: bool = False, undetected: bool = False, download_driver: bool = False) -> None: ...
    def get_headers_with_cookies(self): ...
    def get_cookies(self): ...
    def get_cookies_from_file(self) -> str | None: ...
    def get_cookies_from_browser(self): ...
    def save_cookies_to_path(self, cookies: str): ...
    def test_cookies_valid(self, cookies: str | None) -> bool: ...



from _typeshed import Incomplete
from common_utils.logger import create_logger as create_logger

logged_root_dirs: Incomplete

def get_root_dir(dunder_file: str) -> str | None: ...
def load_config_yaml(file_path: str) -> dict | None: ...
def config_entry(key): ...

logger: Incomplete
ROOT_DIR: Incomplete
CONFIG: Incomplete



import pandas as pd
from _typeshed import Incomplete
from common_utils.config import load_config_yaml as load_config_yaml, secret as secret
from common_utils.logger import create_logger as create_logger

class NotionClient:
    log: Incomplete
    api: Incomplete
    def __init__(self, notion_secret: Incomplete | None = None) -> None: ...
    def get_properties(self, page, property_mapping) -> dict: ...
    def get_database(self, database_id: str, properties_mapping: dict[str, str], filter={}, sort={}) -> pd.DataFrame: ...
    def get_multiple_databases(self, db_config_dict: dict[dict]) -> dict[str, pd.DataFrame]: ...
    def get_page_content(self, page_id: str, parse_mode: str) -> str: ...
    def add_property(self, prop_type, value): ...
    def create_db_entry(self, database_id, properties: Incomplete | None = None, data: Incomplete | None = None, properties_mapping: Incomplete | None = None) -> None: ...



from common_utils.config import ROOT_DIR as ROOT_DIR
from common_utils.logger import create_logger as create_logger
from common_utils.web.cookies_handler import CookiesManager as CookiesManager, LoginData as LoginData, LoginSelectors as LoginSelectors

def get_authenticated_ticktick_headers(cookies_path: str | None = None, username_env: str = 'TICKTICK_EMAIL', password_env: str = 'TICKTICK_PASSWORD', headless: bool = True, undetected: bool = False, download_driver: bool = False): ...



import os
import requests
from datetime import datetime
import pytz
from pydantic import BaseModel, field_validator, model_validator, ConfigDict

from common_utils.logger import create_logger
from common_utils.apis.ticktick.cookies_login import get_authenticated_ticktick_headers
from common_utils.time_utils import get_timestamp_from_offset


class TickTickFocusTime(BaseModel):
    id: str
    type: int
    start_time: str | datetime
    end_time: str | datetime
    status: int
    pause_duration: int
    total_duration: int | None = None

    @field_validator("start_time", "end_time", mode="before")  # noqa
    @classmethod
    def _parse_datetime(cls, value):
        if isinstance(value, str):
            return datetime.fromisoformat(value)
        return value

    @model_validator(mode="after")
    def _set_timezone_and_duration(self):
        tz_name = os.getenv("LOCAL_TIMEZONE", "Europe/Berlin")
        tz = pytz.timezone(tz_name)
        self.start_time = self.start_time.astimezone(tz)
        self.end_time = self.end_time.astimezone(tz)
        self.total_duration = int((self.end_time - self.start_time).total_seconds() / 60)
        return self

    @staticmethod
    def to_camel(field_name: str) -> str:
        """
        Convert a snake_case field name into camelCase.
        E.g. 'checkin_stamp' -> 'checkinStamp'
        """
        parts = field_name.split('_')
        return parts[0] + ''.join(word.capitalize() for word in parts[1:])

    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
        extra="allow"
    )


class TicktickFocusHandler:
    log = create_logger('TickTick Focus Handler')

    def __init__(
            self,
            cookies_path: str | None = None,
            always_raise_exceptions: bool = False,
            headless: bool = True,
            undetected: bool = False,
            download_driver: bool = False,
            username_env: str = 'TICKTICK_EMAIL',
            password_env: str = 'TICKTICK_PASSWORD',
    ):
        self.headers = get_authenticated_ticktick_headers(
            cookies_path=cookies_path,
            username_env=username_env,
            password_env=password_env,
            headless=headless,
            undetected=undetected,
            download_driver=download_driver,
        )
        self.raise_exceptions = always_raise_exceptions

    def get_all_focus_times(
            self,
            to_timestamp: int | None = None,
            days_offset: int | None = None
    ) -> list[TickTickFocusTime] | None:
        if days_offset:
            to_timestamp = get_timestamp_from_offset(days_offset=days_offset)

        timestamp_query_param = f"?to={to_timestamp}" if to_timestamp else ''
        url = f"https://api.ticktick.com/api/v2/pomodoros/timeline{timestamp_query_param}"
        response = requests.get(url, headers=self.headers).json()
        if not isinstance(response, list):
            error_msg = f'No or malformed response to get_all_focus_times: {response}'
            self.log.error(error_msg)
            if self.raise_exceptions:
                raise ValueError(error_msg)
            return None

        focus_times = [TickTickFocusTime(**focus_time_data) for focus_time_data in response]
        return focus_times



if __name__ == '__main__':
    handlers = TicktickFocusHandler()
    focus_times_ = handlers.get_all_focus_times()
    print(focus_times_)


from datetime import datetime
import pygetwindow as gw
import subprocess
import psutil
from time import sleep
from common_utils.logger import create_logger


def close_window_by_process(process_name):
    from pywinauto import Desktop

    windows = Desktop(backend="uia").windows()
    for win in windows:
        pid = win.process_id()
        win_process_name = psutil.Process(pid).name()
        if win_process_name == process_name:
            win.close()
            return True
    return False


def is_program_running(program_name):
    """Check if the program is currently running without showing a console window."""
    try:
        # Set up the startup information for the subprocess
        startupinfo = subprocess.STARTUPINFO()
        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        startupinfo.wShowWindow = subprocess.SW_HIDE

        # Execute 'tasklist' with the configured startupinfo to hide the console window
        processes = subprocess.check_output(["tasklist"], startupinfo=startupinfo, text=True)
        return program_name in processes
    except subprocess.CalledProcessError:
        return False


def start_program(program_path):
    """Starts the program."""
    try:
        startupinfo = subprocess.STARTUPINFO()
        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        startupinfo.wShowWindow = subprocess.SW_HIDE
        subprocess.Popen(program_path, startupinfo=startupinfo)
        print(f"Program started: {program_path}")
    except Exception as e:
        print(f"Failed to start the program: {e}")


class WindowHandler:
    def __init__(self, ignore_windows=None):
        self.log = create_logger("Window Handler")
        self.ignore_windows = (
            ["", "Program Manager", "Windows Input Experience", "Settings"]
            if ignore_windows is None
            else ignore_windows
        )
        self.minimized_windows = []

    def get_user_windows(self):
        """Get all open windows"""
        user_windows = []
        windows = gw.getAllWindows()
        for window in windows:
            if window.title not in self.ignore_windows:
                user_windows.append(window)
        return user_windows

    def minimize_open_windows(self):
        all_windows = self.get_user_windows()
        self.minimized_windows = []
        for win in all_windows:
            if win.title not in self.ignore_windows and win.isMinimized is not True:
                window_info = {
                    "window": win,
                    "is_maximised": win.isMaximized,
                    "is_focused": win.isActive,
                }
                self.minimized_windows.append(window_info)
                win.minimize()
                self.log.debug(f"Minimized window: {window_info}")

    def restore_open_windows(self):
        """Restore the minimized windows, with the focused window restored last"""
        focused_windows = [
            win_info for win_info in self.minimized_windows if win_info["is_focused"]
        ]
        non_focused_windows = [
            win_info for win_info in self.minimized_windows if not win_info["is_focused"]
        ]
        for win_info in non_focused_windows:
            self.log.debug(f"Restoring non-focused window: {win_info['window'].title}")
            win_info["window"].restore()
        for win_info in focused_windows:
            self.log.debug(f"Restoring focused window: {win_info['window'].title}")
            win_info["window"].restore()


if __name__ == "__main__":
    handler = WindowHandler()
    handler.minimize_open_windows()
    print("Windows minimized", handler.minimized_windows)
    sleep(1)
    handler.restore_open_windows()



from _typeshed import Incomplete
from common_utils.config import ROOT_DIR as ROOT_DIR

class SoundHandler:
    default_volume: Incomplete
    def __init__(self, default_volume) -> None: ...
    def play_sound(self, file_path, volume: Incomplete | None = None) -> None: ...



import wmi

IGNORE_ADAPTERS = [
    "Intel(R) Wireless Bluetooth(R) Reset Device",
    "Intel(R) Wireless Bluetooth(R)",
    "Bluetooth",
]


def bluetooth_is_enabled(search_name="Standard Serial over Bluetooth link"):
    """Extendable function to check if bluetooth is enabled."""
    c = wmi.WMI()
    bluetooth_adapter = [
        adapter for adapter in c.Win32_PNPEntity() if "Bluetooth" in str(adapter.Name)
    ]
    adapter_infos = []
    for adapter in bluetooth_adapter:
        adapter_info = {
            "Name": adapter.Name,
            "Status": adapter.Status,
            "PNPDeviceID": adapter.PNPDeviceID,
            "Service": adapter.Service,
        }
        if adapter_info["Name"] not in IGNORE_ADAPTERS:
            adapter_infos.append(adapter_info)
        if search_name in adapter_info["Name"]:
            return True
    return False


if __name__ == "__main__":
    print(f"Bluetooth is enabled: {bluetooth_is_enabled()}")



import logging
from _typeshed import Incomplete

format_str: str
client: Incomplete

class FixedWidthFormatter(logging.Formatter):
    name_length: Incomplete
    def __init__(self, fmt: Incomplete | None = None, datefmt: Incomplete | None = None, name_length: int = 16) -> None: ...
    def format(self, record, name_length: int = 16): ...

def create_logger(name: str, long_date_format: bool = False, name_length: int = 18) -> logging.Logger: ...



from _typeshed import Incomplete
from common_utils.apis.ticktick.cookies_login import get_authenticated_ticktick_headers as get_authenticated_ticktick_headers
from common_utils.logger import create_logger as create_logger
from common_utils.time_utils import get_datetime_now_utc_millisecond as get_datetime_now_utc_millisecond
from common_utils.web.api_request import post_request as post_request
from pydantic import BaseModel
from typing import Literal

class TickTickHabitEntry(BaseModel):
    habit_id: str
    checkin_stamp: int
    goal: float | int
    value: float | int
    status: Literal[0, 1, 2]
    id: str | None
    checkin_time: str | None
    op_time: str | None
    @classmethod
    def init(cls, habit_id: str, date_stamp: int, habit_goal: int, status: Literal[0, 1, 2] | None = None, value: int | float | None = None) -> TickTickHabitEntry: ...
    @staticmethod
    def to_camel(field_name: str) -> str: ...
    model_config: Incomplete

class TicktickHabitHandler:
    status_codes: Incomplete
    url_habits: str
    url_batch_checkin: str
    url_query_checkin: str
    log: Incomplete
    headers: Incomplete
    raise_exceptions: Incomplete
    def __init__(self, cookies_path: str | None = None, always_raise_exceptions: bool = False, headless: bool = True, undetected: bool = False, download_driver: bool = False, username_env: str = 'TICKTICK_EMAIL', password_env: str = 'TICKTICK_PASSWORD') -> None: ...
    def post_checkin(self, habit_name: str, date_stamp: int, status: int | None = None, value: int | None = None, raise_exception: bool = False) -> None: ...
    def get_checkin(self, habit_id: str, date_stamp: int, raise_exception: bool = False) -> TickTickHabitEntry | None: ...
    def get_all_checkins(self, after_stamp: int = 19700101, habit_names: list[str] | str | None = None, raise_exception: bool = False) -> dict[str, list[TickTickHabitEntry]] | None: ...



import requests
import json

from common_utils.logger import create_logger

log = create_logger("Request Helper")


def get_request(url, headers=None, catch_exception=False) -> dict | None:
    """
    Send a POST request to the given URL with the given data and headers.
    """
    if headers is None:
        headers = {"Content-Type": "application/json"}
    try:
        response = requests.get(url, headers=headers)
        return response.json()
    except Exception as e:
        if catch_exception:
            return None
        else:
            raise Exception(f"Error sending POST request to {url}: {e}")


def post_request(url, payload, headers=None, raise_exception=False) -> dict | None:
    """
    Send a POST request to the given URL with the given data and headers.
    """
    if headers is None:
        headers = {"Content-Type": "application/json"}
    try:
        response = requests.post(url, data=json.dumps(payload), headers=headers)
        return response.json()
    except Exception as e:
        if raise_exception:
            raise Exception(f"Error sending POST request to {url}: {e}")
        else:
            return None



from _typeshed import Incomplete
from common_utils.apis.ticktick.cookies_login import get_authenticated_ticktick_headers as get_authenticated_ticktick_headers
from common_utils.logger import create_logger as create_logger
from pydantic import BaseModel

class TickTickTask(BaseModel):
    id: str
    project_id: str
    title: str
    status: int
    priority: int
    deleted: int
    created_time: str
    creator: int
    items: list
    column_id: str | None
    is_all_day: bool | None
    start_date: str | None
    due_date: str | None
    content: str | None
    @staticmethod
    def to_camel(field_name: str) -> str: ...
    model_config: Incomplete

class TicktickTaskHandler:
    log: Incomplete
    url_get_tasks: str
    headers: Incomplete
    raise_exceptions: Incomplete
    def __init__(self, cookies_path: str | None = None, always_raise_exceptions: bool = False, headless: bool = True, undetected: bool = False, download_driver: bool = False, username_env: str = 'TICKTICK_EMAIL', password_env: str = 'TICKTICK_PASSWORD') -> None: ...
    def get_all_tasks(self) -> list[TickTickTask] | None: ...



from _typeshed import Incomplete
from common_utils.logger import create_logger as create_logger
from datetime import datetime as datetime

def close_window_by_process(process_name): ...
def is_program_running(program_name): ...
def start_program(program_path) -> None: ...

class WindowHandler:
    log: Incomplete
    ignore_windows: Incomplete
    minimized_windows: Incomplete
    def __init__(self, ignore_windows: Incomplete | None = None) -> None: ...
    def get_user_windows(self): ...
    def minimize_open_windows(self) -> None: ...
    def restore_open_windows(self) -> None: ...



from _typeshed import Incomplete
from common_utils.web.cookies_handler import CookiesManager as CookiesManager, LoginData as LoginData, LoginSelectors as LoginSelectors

url: str

class BudgetBakerHandler:
    cookies_manager: Incomplete
    cookies: Incomplete
    headers: Incomplete
    def __init__(self) -> None: ...
